<?php

/**
 * $Revision: $
 * $Author: $
 * $Date: $
 * $Id: $
 * @file      trosofortueberweisungorder.php 
 * @link      http://www.tro.net
 * @copyright (C) tronet GmbH 2014
 * @package   modules
 * @addtogroup controllers
 * @extend order
 */

/**
 * Order manager. Arranges user ordering data, checks/validates
 * it, on success stores ordering data to DB.
 *
 * NEW: sets ordernr before finalizing and continues finalizing, if stopped during paymentexecution
 */
class tropaycodeoxorder extends tropaycodeoxorder_parent {

    private $_paycodeUrl;
    private $_paycode;

    protected function _sendOrderByEmail($oUser = null, $oBasket = null, $oPayment = null) {
        ## TRONET ANFANG ##
        // Determine the currency
        $sCurrency = $this->_getCurrency($oBasket);

        // Determine the finale price 
        $fPrice = $this->_getPrice($oBasket);

        $oId = $this->getId();

        $paymentType = $this->getPaymentType()->oxuserpayments__oxpaymentsid->rawValue;
        if ($paymentType == "oxidinvoice" || $paymentType == "oxidpayadvance") {
            $this->setUpPaycode($fPrice, $sCurrency, $oId, $oUser);
        }
        ## TRONET ENDE ##

        return parent::_sendOrderByEmail($oUser, $oBasket, $oPayment);
    }

    /**
     * Function determines the final price, which is required for the paycode
     * 
     * @param oxBasket $oBasket
     * @return float price
     */
    private function _getPrice(oxBasket $oBasket) {

        return $oBasket->getPrice()->getPrice();
    }

    /**
     * Function determines the the currency-code, which is required for the paycode
     * 
     * @param oxBasket $oBasket
     * @return string CurrencyCode
     */
    private function _getCurrency(oxBasket $oBasket) {

        return $oBasket->getBasketCurrency()->name;
    }

    /**
     * Function sets up the paycode for the order-process. 
     * @param float $price
     * @param string $oxid
     * @param oUser $oUser
     */
    public function setUpPaycode($price, $curr, $oxid, $oUser) {
        require_once( oxregistry::getConfig()->getModulesDir() . 'tronet/tropaycode/library/payment/sofortLibPaycode.inc.php');


        $helper = oxNew('trohelper');
        $configKey = $helper->getProperty('sTroPaycodeConfigKey');

        $SofortLibPaycode = new SofortLibPaycode($configKey);

        // Placeholder for values from the backend
        $aPlaceHolder = array(
            '[BSTNR]' => $this->oxorder__oxordernr->value,
            '[KNR]' => $oUser->oxuser__oxcustnr->value,
            '[KNAME]' => $oUser->oxuser__oxlname->value,
            '[DATUM]' => date('d.m.Y'),
            '[PRICE]' => round($price),
        );

        $oShop = oxregistry::getConfig()->getActiveShop();

        // Replace umlauts for the reason, so we don't have any unwanted chars in there
        $aReason[0] = $this->_replaceUmlauts(utf8_decode($oShop->oxshops__oxname->value));
        $aReason[1] = $this->_replaceUmlauts(utf8_decode(str_replace(array_keys($aPlaceHolder), array_values($aPlaceHolder), oxregistry::getConfig()->getConfigParam('sTroPaycodeReason'))));

        $aModuleVersions = oxregistry::getConfig()->getConfigParam('aModuleVersions');
        $paycodeVersion = $aModuleVersions['tropaycode'];

        // Fill the paycode with the specific values
        $SofortLibPaycode->setAmount($price)
                ->setVersion('Oxid_Paycode_Version_' . $paycodeVersion)
                ->setCurrencyCode($curr)
                ->setReason($aReason[0], $aReason[1])
                ->setSuccessUrl(oxregistry::getConfig()->getSslShopUrl() . '?cl=tropaycode&fnc=updateStatus&transactionid=-TRANSACTION-')
                ->setSuccessLinkRedirect("")
                ->setAbortUrl('http://www.sofort.com')
                ->setNotificationUrl(oxregistry::getConfig()->getSslShopUrl() . '?cl=tropaycode&fnc=notifyStatus&transactionid=-TRANSACTION-')
                ->setUserVariable($oxid);

        // create a paycode: If there are any errors, show the error
        $SofortLibPaycode->createPaycode();
        if ($SofortLibPaycode->isError()) {
            //Sofort-API didn't accept the data
            echo $SofortLibPaycode->getError();
        }
        $this->_paycodeUrl = $SofortLibPaycode->getPaycodeUrl();
        $this->_paycode = $SofortLibPaycode->getPaycode();

        // Store some data in the database
//        $logger = oxNew('tropaycodesorder');
//        $logger->tropaycodesorder__paycode = new oxField($this->_paycode);
//        
//        $logger->tropaycodesorder__ordernum = new oxField($oxid);
//        $logger->tropaycodesorder__timestamp = new oxField(date('Y-m-d H:i:s'));
//        
//        $logger->save();

        $logger = oxNew('tropaycodelog');
        $logger->tropaycodelog__trooxidorder = new oxField($oxid);
        $logger->tropaycodelog__paycode = new oxField($this->_paycode);
        $logger->tropaycodelog__status = new oxField("generated");
        $logger->tropaycodelog__timestamp = new oxField(date('Y-m-d H:i:s'));

        $logger->save();
    }

    /**
     * Function returns the paycode url to SOFORT AG with the specific paycode
     * @return string paycodeurl
     */
    public function getPaycodeUrl() {
        return $this->_paycodeUrl;
    }

    /**
     * Function returns the paycode alone
     * @return string paycode
     */
    public function getPaycode() {
        return $this->_paycode;
    }

    /**
     * Function replaces special chars to some equivalent symbols or typos
     * @param string $s
     * @return string cleaned Inputstring
     */
    private function _replaceUmlauts($s) {
        $search = array("", "", "", "", "", "", "", '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '', '');
        $replace = array("Ae", "Oe", "Ue", "ae", "oe", "ue", "ss", 'A', 'A', 'A', 'a', 'a', 'a', 'E', 'E', 'E', 'e', 'e', 'e', 'I', 'I', 'I', 'i', 'i', 'i', 'O', 'O', 'O', 'o', 'o', 'o', 'U', 'U', 'U', 'u', 'u', 'u');
        return str_replace($search, $replace, $s);
    }

}
